@@ -18,11 +18,30 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import urwid
-from logging import debug, info, warning, error
+import logging as log
 import encodings
 utf8decode = lambda s: encodings.codecs.utf_8_decode(s)[0]
 
 from urwid.util import is_mouse_press #XXX: is_mouse_press is not included in urwid in 1.0.0
+from .keys import action_key_map as a_key
+
+FOCUS_KEYS = (a_key['FOCUS_SWITCH'], a_key['FOCUS_UP'], a_key['FOCUS_DOWN'])
+
+
+def getFocusDirection(key, inversed=False):
+    """Return direction and rotate boolean depending on key
+    @param key: one of FOCUS_KEYS
+    @param inversed: inverse directions if True
+    @return (tuple): (direction, rotate) where
+        - direction is 1 or -1
+        - rotate is False if we should stop at the begin/end of the widgets list
+    """
+    if not inversed:
+        direction = 1 if key in (a_key['FOCUS_SWITCH'], a_key['FOCUS_UP']) else -1
+    else:
+        direction = -1 if key in (a_key['FOCUS_SWITCH'], a_key['FOCUS_UP']) else 1
+    rotate = key == a_key['FOCUS_SWITCH']
+    return direction, rotate
 
 
 class AdvancedEdit(urwid.Edit):
@@ -50,21 +69,21 @@ class AdvancedEdit(urwid.Edit):
 
     def keypress(self, size, key):
         #TODO: insert mode is not managed yet
-        if key == 'ctrl a':
+        if key == a_key['EDIT_HOME']:
             key = 'home'
-        elif key == 'ctrl e':
+        elif key == a_key['EDIT_END']:
             key = 'end'
-        elif key == 'ctrl k':
+        elif key == a_key['EDIT_DELETE_TO_END']:
             self._delete_highlighted()
             self.set_edit_text(self.edit_text[:self.edit_pos])
-        elif key == 'ctrl w':
+        elif key == a_key['EDIT_DELETE_LAST_WORD']:
             before = self.edit_text[:self.edit_pos]
             pos = before.rstrip().rfind(" ")+1
             self.set_edit_text(before[:pos] + self.edit_text[self.edit_pos:])
             self.set_edit_pos(pos)
-        elif key == 'enter':
+        elif key == a_key['EDIT_ENTER']:
             self._emit('click')
-        elif key == 'shift tab':
+        elif key == a_key['EDIT_COMPLETE']:
             try:
                 before = self.edit_text[:self.edit_pos]
                 if self.completion_data:
@@ -154,7 +173,7 @@ class ModalEdit(AdvancedEdit):
         super(ModalEdit, self).setCompletionMethod(lambda text,data: callback(text, data, self._mode))
 
     def keypress(self, size, key):
-        if key == 'esc':
+        if key == a_key['MODAL_ESCAPE']:
             self.mode = "NORMAL"
             return
         if self._mode == 'NORMAL' and key in self._modes:
@@ -184,28 +203,30 @@ class SurroundedText(urwid.Widget):
         return urwid.Text(render_text)
 
 
-class SelectableText(urwid.WidgetWrap):
+class AlwaysSelectableText(urwid.WidgetWrap):
     """Text which can be selected with space"""
     signals = ['change']
 
     def __init__(self, text, align='left', header='', focus_attr='default_focus', selected_text=None, selected=False, data=None):
-        """@param text: same as urwid.Text's text parameter
+        """
+        @param text: same as urwid.Text's text parameter
         @param align: same as urwid.Text's align parameter
         @select_attr: attrbute to use when selected
-        @param selected: is the text selected ?"""
+        @param selected: is the text selected ?
+        """
         self.focus_attr = focus_attr
         self.__selected = False
         self.__was_focused = False
         self.header = header
-        self.default_txt = text
+        self.text = text
         urwid.WidgetWrap.__init__(self, urwid.Text("",align=align))
         self.setSelectedText(selected_text)
         self.setState(selected)
 
     def getValue(self):
-        if isinstance(self.default_txt,basestring):
-            return self.default_txt
-        list_attr = self.default_txt if isinstance(self.default_txt, list) else [self.default_txt]
+        if isinstance(self.text,basestring):
+            return self.text
+        list_attr = self.text if isinstance(self.text, list) else [self.text]
         txt = ""
         for attr in list_attr:
             if isinstance(attr,tuple):
@@ -220,7 +241,7 @@ class SelectableText(urwid.WidgetWrap):
 
     def set_text(self, text):
         """/!\ set_text doesn't change self.selected_txt !"""
-        self.default_txt = text
+        self.text = text
         self.setState(self.__selected,invisible=True)
 
     def setSelectedText(self, text=None):
@@ -232,10 +253,9 @@ class SelectableText(urwid.WidgetWrap):
         if self.__selected:
             self.setState(self.__selected)
 
-
     def __set_txt(self):
         txt_list = [self.header]
-        txt = self.selected_txt if self.__selected else self.default_txt
+        txt = self.selected_txt if self.__selected else self.text
         if isinstance(txt,list):
             txt_list.extend(txt)
         else:
@@ -262,7 +282,7 @@ class SelectableText(urwid.WidgetWrap):
         return True
 
     def keypress(self, size, key):
-        if key==' ' or key=='enter':
+        if key in (a_key['TEXT_SELECT'], a_key['TEXT_SELECT2']):
             self.setState(not self.__selected)
         else:
             return key
@@ -299,6 +319,13 @@ class SelectableText(urwid.WidgetWrap):
         return self._w.render(size, focus)
 
 
+class SelectableText(AlwaysSelectableText):
+    """Like AlwaysSelectableText but not selectable when text is empty"""
+
+    def selectable(self):
+        return bool(self.text)
+
+
 class ClickableText(SelectableText):
     signals = SelectableText.signals + ['click']
 
@@ -389,6 +416,36 @@ class ListOption(unicode):
         return [(ListOption(option)) for option in options]
 
 
+class UnselectableListBox(urwid.ListBox):
+    """List box that can be unselectable if all widget are unselectable and visible"""
+
+    def __init__(self, body):
+        super(UnselectableListBox, self).__init__(body)
+        self.__size_cache = None
+
+    def selectable(self):
+        """Selectable that return False if everything is visible and nothing is selectable"""
+        if self.__size_cache is None:
+            return self._selectable
+        middle, top, bottom = self.calculate_visible(self.__size_cache, self.__focus_cache)
+        if top is None or bottom is None:
+            return True
+        if top[0] or bottom[0]:
+            # if not everything is visible, we can select
+            return True
+        for wid in self.body:
+            # if any widget is selectable, we can select
+            if wid.selectable():
+                return True
+        return False
+
+    def render(self, size, focus=False):
+        """Call ListBox render, but keep size and focus in cache"""
+        self.__size_cache = size
+        self.__focus_cache = focus
+        return super(UnselectableListBox, self).render(size, focus)
+
+
 class GenericList(urwid.WidgetWrap):
     signals = ['click','change']
 
@@ -497,17 +554,44 @@ class GenericList(urwid.WidgetWrap):
         self._emit('change')
         self.first_display = False
 
-    def selectValue(self, value):
-        """Select the first item which has the given value"""
+    def selectValue(self, value, move_focus=True):
+        """Select the first item which has the given value.
+
+        @param value
+        @param move_focus (boolean): True to move the focus on the selected value,
+                                     False to leave the focus position unchanged.
+        """
         self.unselectAll()
         idx = 0
         for widget in self.content:
             if widget.getValue() == value:
                 widget.setState(True)
-                self.list_box.set_focus(idx)
+                if move_focus:
+                    self.list_box.focus_position = idx
                 return
             idx+=1
 
+    def selectValues(self, values, move_focus=True):
+        """Select all the given values.
+
+        @param values [set, list]
+        @param move_focus (boolean): True to move the focus on the last selected value,
+                                     False to leave the focus position unchanged.
+        """
+        if self.single:
+            if values:
+                self.selectValue(values[-1], move_focus)
+            return
+        self.unselectAll()
+        for value in values:
+            idx = 0
+            for widget in self.content:
+                if widget.getValue() == value:
+                    widget.setState(True)
+                    if move_focus:
+                        self.list_box.focus_position = idx
+                idx += 1
+
 
 class List(urwid.Widget):
     """FlowWidget list, same arguments as GenericList, with an additional one 'max_height'"""
@@ -552,8 +636,11 @@ class List(urwid.Widget):
     def changeValues(self, new_values):
         return self.genericList.changeValues(new_values)
 
-    def selectValue(self, value):
-        return self.genericList.selectValue(value)
+    def selectValue(self, value, move_focus=True):
+        return self.genericList.selectValue(value, move_focus)
+
+    def selectValues(self, values, move_focus=True):
+        return self.genericList.selectValues(values, move_focus)
 
     def render(self, size, focus=False):
         return self.displayWidget(size, focus).render(size, focus)
@@ -593,6 +680,8 @@ class NotificationBar(urwid.WidgetWrap):
             self.progress.set_text('')
         else:
             self.progress.set_text(('notifs','%02i%%' % percentage))
+            if self.columns.focus != self.progress:
+                self.columns.focus_position = len(self.columns.contents)-1
         self._emit('change')
 
     def addPopUp(self, pop_up_widget):
@@ -621,6 +710,7 @@ class NotificationBar(urwid.WidgetWrap):
             self.notifs.remove(found)
             self.message.set_text(('notifs',found[1]))
             self.__modQueue()
+            self.focus_possition = 1
         else:
             self.message.set_text('')
             self._emit('change')
@@ -642,7 +732,7 @@ class NotificationBar(urwid.WidgetWrap):
         return not bool(self.notifs)
 
     def canHide(self):
-        """Return True if there is now important information to show"""
+        """Return True if there is no important information to show"""
         return self.isQueueEmpty() and not self.message.get_text() and not self.progress.get_text()
 
 
@@ -665,10 +755,10 @@ class MenuBox(urwid.WidgetWrap):
         return self.selected
 
     def keypress(self, size, key):
-        if key=='up':
+        if key==a_key['MENU_BOX_UP']:
             if self.listBox.get_focus()[1] == 0:
                 self.parent.keypress(size, key)
-        elif key=='left' or key=='right':
+        elif key in (a_key['MENU_BOX_LEFT'], a_key['MENU_BOX_RIGHT']):
             self.parent.keypress(size,'up')
             self.parent.keypress(size,key)
         return super(MenuBox,self).keypress(size,key)
@@ -726,9 +816,9 @@ class Menu(urwid.WidgetWrap):
         self.loop.widget = urwid.Overlay(urwid.AttrMap(menu_box,'menubar'),self.save_bottom,('fixed left', columns),max_len+2,('fixed top',1),None)
 
     def keypress(self, size, key):
-        if key == 'down':
+        if key == a_key['MENU_DOWN']:
             key = 'enter'
-        elif key == 'up':
+        elif key == a_key['MENU_UP']:
             if  self.save_bottom:
                 self.loop.widget = self.save_bottom
                 self.save_bottom = None
@@ -742,8 +832,9 @@ class Menu(urwid.WidgetWrap):
                 callback((category, item))
         return key
 
-    def addMenu(self, category, item, callback, shortcut=None):
-        """Add a menu item, create the category if new
+    def addMenu(self, category, item=None, callback=None, shortcut=None):
+        """Create the category if new and add a menu item (if item is not None).
+
         @param category: category of the menu (e.g. File/Edit)
         @param item: menu item (e.g. new/close/about)
         @callback: method to call when item is selected"""
@@ -754,6 +845,8 @@ class Menu(urwid.WidgetWrap):
                                    left_border = ('menubar',"[ "),
                                    right_border = ('menubar'," ]"))
             self._w.base_widget.addWidget(button,button.getSize())
+        if not item:
+            return
         self.menu[category].append((item, callback))
         if shortcut:
             assert(shortcut not in self.shortcuts.keys())
@@ -768,7 +861,7 @@ class Menu(urwid.WidgetWrap):
                 callback = menu_item[1]
                 break
         if callback:
-            self.keypress(None,'up')
+            self.keypress(None, a_key['MENU_UP'])
             callback((category, item))
 
     def onCategoryClick(self, button):
@@ -794,7 +887,7 @@ class MenuRoller(urwid.WidgetWrap):
             name,menu = menu_tuple
             self.addMenu(name, menu)
 
-    def __showSelected(self):
+    def _showSelected(self):
         """show menu selected"""
         name_txt = u'\u21c9 '+self.name_list[self.selected]+u' \u21c7 '
         current_name = ClickableText(name_txt)
@@ -805,15 +898,19 @@ class MenuRoller(urwid.WidgetWrap):
         self.columns.contents[1] = (current_menu, ('weight', 1, False))
 
     def keypress(self, size, key):
-        if key=='up':
-            if self.columns.get_focus_column()==0 and self.selected > 0:
-                self.selected -= 1
-                self.__showSelected()
-        elif key=='down':
-            if self.columns.get_focus_column()==0 and self.selected < len(self.name_list)-1:
-                self.selected += 1
-                self.__showSelected()
-        elif key=='right':
+        if key==a_key['MENU_ROLLER_UP']:
+            if self.columns.get_focus_column()==0:
+                if self.selected > 0:
+                    self.selected -= 1
+                    self._showSelected()
+                return
+        elif key==a_key['MENU_ROLLER_DOWN']:
+            if self.columns.get_focus_column()==0:
+                if self.selected < len(self.name_list)-1:
+                    self.selected += 1
+                    self._showSelected()
+                return
+        elif key==a_key['MENU_ROLLER_RIGHT']:
             if self.columns.get_focus_column()==0 and \
                 (isinstance(self.columns.contents[1][0], urwid.Text) or \
                 self.menus[self.name_list[self.selected]].getMenuSize()==0):
@@ -827,7 +924,7 @@ class MenuRoller(urwid.WidgetWrap):
             self.name_list.append(name)
         self.menus[name] = menu
         if self.name_list[self.selected] == name:
-            self.__showSelected() #if we are on the menu, we update it
+            self._showSelected() #if we are on the menu, we update it
 
     def removeMenu(self, name):
         if name in self.name_list:
@@ -835,7 +932,7 @@ class MenuRoller(urwid.WidgetWrap):
         if name in self.menus.keys():
             del self.menus[name]
         self.selected = 0
-        self.__showSelected()
+        self._showSelected()
 
     def checkShortcuts(self, key):
         for menu in self.name_list:
@@ -870,7 +967,7 @@ class GenericDialog(urwid.WidgetWrap):
         if buttons:
             buttons_flow = urwid.GridFlow(buttons, max([len(button.get_label()) for button in buttons])+4, 1, 1, 'center')
         body_content = urwid.SimpleListWalker(widgets_lst)
-        frame_body = urwid.ListBox(body_content)
+        frame_body = UnselectableListBox(body_content)
         frame = FocusFrame(frame_body, frame_header, buttons_flow if buttons else None, 'footer' if buttons else 'body')
         decorated_frame = urwid.LineBox(frame)
         urwid.WidgetWrap.__init__(self, decorated_frame)
@@ -885,16 +982,20 @@ class InputDialog(GenericDialog):
         instr_wid = urwid.Text(instrucions+':')
         edit_box = AdvancedEdit(edit_text=default_txt)
         GenericDialog.__init__(self, [instr_wid,edit_box], title, style, ok_value=edit_box, **kwargs)
-        self._w.base_widget.set_focus('body')
+        self._w.base_widget.focusposition = 'body'
 
 
 class ConfirmDialog(GenericDialog):
     """Dialog with buttons for confirm or cancel an action"""
 
-    def __init__(self, title, style=None, **kwargs):
+    def __init__(self, title, message=None, style=None, **kwargs):
         if style is None:
             style = ['YES/NO']
-        GenericDialog.__init__(self, [], title, style, **kwargs)
+        GenericDialog.__init__(self,
+                               [urwid.Text(message, 'center')] if message is not None else [],
+                               title,
+                               style,
+                               **kwargs)
 
 
 class Alert(GenericDialog):
@@ -903,7 +1004,6 @@ class Alert(GenericDialog):
     def __init__(self, title, message, style=['OK'], **kwargs):
         GenericDialog.__init__(self, [urwid.Text(message, 'center')], title, style, ok_value=None, **kwargs)
 
-
 ## CONTAINERS ##
 
 class ColumnsRoller(urwid.Widget):
@@ -918,7 +1018,7 @@ class ColumnsRoller(urwid.Widget):
     def addWidget(self, widget, width):
         self.widget_list.append((width,widget))
         if len(self.widget_list) == 1:
-            self.set_focus(0)
+            self.focus_position = 0
 
     def getStartCol(self, widget):
         """Return the column of the left corner of the widget"""
@@ -936,12 +1036,12 @@ class ColumnsRoller(urwid.Widget):
             return False
 
     def keypress(self, size, key):
-        if key=='left':
+        if key==a_key['COLUMNS_ROLLER_LEFT']:
             if self.focus_column>0:
                 self.focus_column-=1
                 self._invalidate()
                 return
-        if key=='right':
+        if key==a_key['COLUMNS_ROLLER_RIGHT']:
             if self.focus_column<len(self.widget_list)-1:
                 self.focus_column+=1
                 self._invalidate()
@@ -954,7 +1054,12 @@ class ColumnsRoller(urwid.Widget):
         """Return selected widget"""
         return self.widget_list[self.focus_column][1]
 
-    def set_focus(self, idx):
+    @property
+    def focus_position(self):
+        return self.focus_column
+
+    @focus_position.setter
+    def focus_position(self, idx):
         if idx>len(self.widget_list)-1:
             idx = len(self.widget_list)-1
         self.focus_column = idx
@@ -994,10 +1099,10 @@ class ColumnsRoller(urwid.Widget):
         if is_mouse_press(event) and button == 1:
             _prev,_next,start_wid,end_wid,cols_left = self.__calculate_limits(size)
             if x==0 and _prev:
-                self.keypress(size,'left')
+                self.keypress(size, a_key['COLUMNS_ROLLER_LEFT'])
                 return True
             if x==maxcol-1 and _next:
-                self.keypress(size,'right')
+                self.keypress(size, a_key['COLUMNS_ROLLER_RIGHT'])
                 return True
 
             current_pos = 1 if _prev else 0
@@ -1019,7 +1124,7 @@ class ColumnsRoller(urwid.Widget):
 
     def render(self, size, focus=False):
         if not self.widget_list:
-            return SolidCanvas(" ", size[0], 1)
+            return urwid.SolidCanvas(" ", size[0], 1)
 
         _prev,_next,start_wid,end_wid,cols_left = self.__calculate_limits(size)
 
@@ -1040,44 +1145,102 @@ class ColumnsRoller(urwid.Widget):
         return urwid.CanvasJoin(render)
 
 
-class FocusFrame(urwid.Frame):
-    """Frame which manage 'tab' key"""
+class FocusPile(urwid.Pile):
+    """A Pile Widget which manage SàT Focus keys"""
+    _focus_inversed = False
 
     def keypress(self, size, key):
-        ret = urwid.Frame.keypress(self, size, key)
+        ret = super(FocusPile, self).keypress(size, key)
         if not ret:
             return
 
-        if key == 'tab':
-            focus_list = ('header','body','footer')
-            focus_idx = focus_list.index(self.focus_part)
-            for i in range(2):
-                focus_idx = (focus_idx + 1) % len(focus_list)
-                focus_name = focus_list[focus_idx]
-                widget = getattr(self,'_'+focus_name)
-                if widget!=None and widget.selectable():
-                    self.set_focus(focus_name)
+        if key in FOCUS_KEYS:
+            direction, rotate = getFocusDirection(key, inversed = self._focus_inversed)
+            max_pos = len(self.contents) - 1
+            new_pos = self.focus_position + direction
+            if rotate:
+                if new_pos > max_pos:
+                    new_pos = 0
+                elif new_pos < 0:
+                    new_pos = max_pos
+            try:
+                self.focus_position = new_pos
+            except IndexError:
+                pass
+
+        return key
+
+
+class FocusFrame(urwid.Frame):
+    """Frame-like which manage SàT Focus Keys"""
+    ordered_positions = ('footer', 'body', 'header')
+
+    def keypress(self, size, key):
+        ret = super(FocusFrame, self).keypress(size, key)
+        if not ret:
+            return
+
+        if key in FOCUS_KEYS:
+            direction, rotate = getFocusDirection(key)
+
+            positions = [pos for pos in self.ordered_positions if pos in self]
+            selectables = [pos for pos in positions if self.contents[pos][0].selectable()] # keep positions which exists and have a selectable widget
+            if not selectables:
+                # no widget is selectable, we just return
+                return
+            idx = selectables.index(self.focus_position) + direction
+            if not rotate and (idx < 0 or idx >= len(selectables)):
+                # if we don't rotate, we stay where we are on the first and last position
+                return
+            try:
+                self.focus_position = selectables[idx]
+            except IndexError:
+                # happen if idx > len(selectables)
+                self.focus_position = selectables[0]
+            return
 
         return ret
 
+    def get_cursor_coords(self, size):
+        """Return the cursor coordinates of the focus widget."""
+        if not self.selectable():
+            return None
+        if not hasattr(self.focus, 'get_cursor_coords'):
+            return None
+        maxcol, maxrow = size
+        try:
+            if self.focus_position != 'body':
+                # only body is a box widget
+                size = (maxcol,)
+            col, row = self.focus.get_cursor_coords(size)
+        except TypeError:
+            return None
+        if self.focus_position == 'header':
+            return (col, row)
+        if self.focus_position == 'body':
+            header_rows = self.header.rows((maxcol,))
+            return (col, row + header_rows)
+        if self.focus_position == 'footer':
+            footer_rows = self.footer.rows((maxcol,))
+            return (col, row + (maxrow - footer_rows))
+        raise Exception('This line should not be reached')
+
 
 class TabsContainer(urwid.WidgetWrap):
+    """ Container which can contain multiple box widgets associated to named tabs """
     signals = ['click']
 
     def __init__(self):
         self._current_tab = None
         self._buttons_cont = ColumnsRoller()
         self.tabs = []
-        self.__frame = FocusFrame(urwid.Filler(urwid.Text('')),urwid.Pile([self._buttons_cont,urwid.Divider(u"─")]))
-        urwid.WidgetWrap.__init__(self, self.__frame)
+        self._frame = FocusFrame(urwid.Filler(urwid.Text('')),urwid.Pile([self._buttons_cont,urwid.Divider(u"─")]))
+        urwid.WidgetWrap.__init__(self, self._frame)
 
     def keypress(self, size, key):
-        if key=='tab':
-            self._w.keypress(size,key)
-            return
         return self._w.keypress(size,key)
 
-    def __buttonClicked(self, button, invisible=False):
+    def _buttonClicked(self, button, invisible=False):
         """Called when a button on the tab is changed,
         change the page
         @param button: button clicked
@@ -1087,9 +1250,9 @@ class TabsContainer(urwid.WidgetWrap):
             if tab[0] == tab_name:
                 break
         if tab[0] != tab_name:
-            error(_("INTERNAL ERROR: Tab not found"))
+            log.error(_("INTERNAL ERROR: Tab not found"))
             assert(False)
-        self.__frame.body = tab[1]
+        self._frame.body = tab[1]
         button.set_label(('title',button.get_label()))
         if self._current_tab:
             self._current_tab.set_label(self._current_tab.get_label())
@@ -1097,27 +1260,32 @@ class TabsContainer(urwid.WidgetWrap):
         if not invisible:
             self._emit('click')
 
-    def __appendButton(self, name):
+    def _appendButton(self, name):
         """Append a button to the frame header,
         and link it to the page change method"""
-        button = CustomButton(name, self.__buttonClicked, left_border = '', right_border=' | ')
+        button = CustomButton(name, self._buttonClicked, left_border = '', right_border=' | ')
         self._buttons_cont.addWidget(button, button.getSize())
         if len(self._buttons_cont.widget_list) == 1:
             #first button: we set the focus and the body
-            self._buttons_cont.set_focus(0)
-            self.__buttonClicked(button,True)
+            self._buttons_cont.focus_position = 0
+            self._buttonClicked(button,True)
 
     def addTab(self,name,content=None):
         """Add a page to the container
         @param name: name of the page (what appear on the tab)
-        @param content: content of the page
+        @param content: content of the page:
+            - if None create and empty Listbox
+            - if it is a list instance, create a ListBox with the list in a body
+            - else it must be a box widget which will be used instead of the ListBox
         @return: ListBox (content of the page)"""
-        if content is None:
-            content = []
-        listbox = urwid.ListBox(urwid.SimpleListWalker(content))
-        self.tabs.append([name,listbox])
-        self.__appendButton(name)
-        return listbox
+        if content is None or isinstance(content, list):
+            tab = urwid.ListBox(urwid.SimpleListWalker(content or []))
+        else:
+            tab = content
+
+        self.tabs.append([name, tab])
+        self._appendButton(name)
+        return tab
 
     def addFooter(self, widget):
         """Add a widget on the bottom of the tab (will be displayed on all pages)
@@ -1217,7 +1385,7 @@ class TableContainer(urwid.WidgetWrap):
                 try:
                     options[opt] = tuple(options[opt])
                 except TypeError:
-                    warning('[%s] option is not a tuple' % opt)
+                    log.warning('[%s] option is not a tuple' % opt)
                     options[opt] = ()
         self._options = options
         self._dividechars = dividechars
@@ -1275,6 +1443,8 @@ class TableContainer(urwid.WidgetWrap):
 
         if not columns.selectable() and columns.contents[-1][0].base_widget.selectable():
             columns.focus_position = len(columns.contents)-1
+        if not self.selectable() and columns.selectable():
+            pile.focus_position = len(pile.contents) - 1
         self._idx += 1
 
     def setRowIndex(self, idx):
